using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using System.IO;
using com.epson.pos.driver;

namespace Program05
{
    public partial class Form1 : Form
    {

        // Constant variables holding the printer names.
        private const string PRINTER_NAME_SLIP = "EPSON TM-H6000VI Slip";
        private const string PRINTER_NAME_ENDORSE = "EPSON TM-H6000VI Endorse";
        
        // Bit masks for Type ID acquired through GetType
        private const byte BITMASK_MICREQUIPPED = 0x0008;
        private const byte BITMASK_WITHMICR = 0x0008;

        // Variables/Objects.
        private StatusAPI objAPI = new StatusAPI();

        Font printFont;

        Boolean isPrintFinish;
        Boolean isReadFinish;
        ASB printStatus;

        String MICRdata;
        MICRStatus MICRstatus;
        MICRDetail MICRdetail;
        
        Boolean isInitialized		 = false;
	    Boolean needCloseMonPrinter	 = false;
        Boolean needCancelStatusBack = false;
        Boolean needMICRCancelReadBack = false;

        public Form1()
        {
            InitializeComponent();
        }

        // The executed function when the Start button is clicked.
        // The sequence of the execution is:
        // MICR scanning => Print endorse => Print slip
        private void cmdStart_Click(object sender, EventArgs e)
        {
            if (StartOperation())
            {
                if (ReadMICR())
                {
                    if (PrintEndorse())
                    {
                        PrintSlip();
                    }
                }
            }
            else
            {
                EndOperation();
            }
        }

        // The executed function when the Exit button is clicked.
        private void cmdClose_Click(object sender, EventArgs e)
        {
            EndOperation();
            Close();
        }

        /*****************************************************************************
         * M I C R    C H E C K    R E A D I N G                                     *
         *****************************************************************************/
        private Boolean ReadMICR()
        {
            Boolean retVal = true;

            this.isReadFinish = false;

             // Start check reading.
            if (this.objAPI.MICRReadCheck(MicrFont.MICR_FONT_E13B, 0) != ErrorCode.SUCCESS)
            {
                return false;
            }

            // Wait until callback function will say that the task is done.
            while (!this.isReadFinish)
            {
                System.Threading.Thread.Sleep(500);
            }

            // Error check.
            if ((this.MICRstatus & MICRStatus.ABNORMAL_COMPLETION) != 0)
            {
                retVal = false;
            }
            else
            {
                // Reload the check.
                if (this.objAPI.MICRLoadCheck() != ErrorCode.SUCCESS)
                {
                    retVal = false;
                }
            }
            
            // Display the string data.
            this.txtData.Text = this.MICRdata;
            
            if (retVal)
            {
                MessageBox.Show("MICR reading complete.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Error occurred during MICR reading.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
                this.objAPI.MICREjectCheck();
            }

            return retVal;
        }

        /***************************************************************************
         * E N D O R S E    P R I N T I N G                                        *
         ***************************************************************************/
        private Boolean PrintEndorse()
        {
            bool retVal = true;
            PrintDocument printEndorse;

            this.isPrintFinish = false;

            printEndorse = new PrintDocument();
            printEndorse.PrintPage += new PrintPageEventHandler(PrintEndorse_PrintPage);

            // Use the endorse driver to endorsement printing.
            printEndorse.PrinterSettings.PrinterName = PRINTER_NAME_ENDORSE;

            if (printEndorse.PrinterSettings.IsValid)
            {
                this.printFont = new Font("Consolas", 11, FontStyle.Regular, GraphicsUnit.Point);

                printEndorse.DocumentName = "Endorse test";
                // Start printing.
                printEndorse.Print();

                 // Wait until callback function will say that the task is done.
                while (!this.isPrintFinish)
                {
                    System.Threading.Thread.Sleep(500);
                }

                // Display the status/error message.
                DisplayStatusMessage();
            }
            else
            {
                MessageBox.Show("Printer is not available.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
                this.objAPI.MICREjectCheck();
                retVal = false;
            }
            return retVal;
        }

        /**************************************************************************
         * S L I P    P R I N T I N G                                             *
         **************************************************************************/
        private Boolean PrintSlip()
        {
            bool retVal = true;
            PrintDocument printSlip;

            this.isPrintFinish = false;

            printSlip = new PrintDocument();
            printSlip.PrintPage += new PrintPageEventHandler(PrintSlip_PrintPage);

            // Use the slip driver to front printing.
            printSlip.PrinterSettings.PrinterName = PRINTER_NAME_SLIP;

            if (printSlip.PrinterSettings.IsValid)
            {
                this.printFont = new Font("Consolas", 11, FontStyle.Regular, GraphicsUnit.Point);

                printSlip.DocumentName = "Slip test";
                // Start printing.
                printSlip.Print();

                // Wait until callback function will say that the task is done.
                while (!this.isPrintFinish)
                {
                    System.Threading.Thread.Sleep(500);
                }

                // Display the status/error message.
                DisplayStatusMessage();
            }
            else
            {
                MessageBox.Show("Printer is not available.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
                this.objAPI.MICREjectCheck();
                retVal = false;
            }
            return retVal;
        }

        private void DisplayStatusMessage()
        {
            if (this.printStatus.ToString().Contains(ASB.ASB_PRINT_SUCCESS.ToString()))
                MessageBox.Show("Printing complete.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else if (this.printStatus.ToString().Contains(ASB.ASB_NO_RESPONSE.ToString()))
                MessageBox.Show("No response.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else if (this.printStatus.ToString().Contains(ASB.ASB_COVER_OPEN.ToString()))
                MessageBox.Show("Cover is open.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else if (this.printStatus.ToString().Contains(ASB.ASB_AUTOCUTTER_ERR.ToString()))
                MessageBox.Show("Autocutter error occurred.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else if (this.printStatus.ToString().Contains(ASB.ASB_RECEIPT_END.ToString()))
                MessageBox.Show("Roll paper end sensor: paper not present.", "Program05", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        // The event handler function when printEndorse.Print is called.
        // This is where the actual printing of sample data to the Endorse printer is made.
        private void PrintEndorse_PrintPage(object sender, PrintPageEventArgs e)
        {
            e.Graphics.PageUnit = GraphicsUnit.Point;
            e.Graphics.DrawString("Endorse printing..", this.printFont, Brushes.Black, 15, 10);
            e.HasMorePages = false;
        }

        // The event handler function when printSlip.Print is called.
        // This is where the actual printing of sample data to the Slip printer is made.
        private void PrintSlip_PrintPage(object sender, PrintPageEventArgs e)
        {
            e.Graphics.PageUnit = GraphicsUnit.Point;
            e.Graphics.DrawString("Slip printing..", this.printFont, Brushes.Black, 480, 80);
            e.HasMorePages = false;
        }

        private Boolean StartOperation()
        {
            Boolean retVal = true;

            if (this.isInitialized)
            {
                return retVal;
            }

            // Open a printer status monitor for the selected printer.
            if (this.objAPI.OpenMonPrinter(OpenType.TYPE_PRINTER, PRINTER_NAME_SLIP) != ErrorCode.SUCCESS)
            {
                MessageBox.Show("Failed to open printer status monitor.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
                retVal = false;
            }
            else
            {
                this.needCloseMonPrinter = true;
            }

            // Declare the variables to store data read from MICR.
            if (retVal)
            {
                byte typeA = 0;
                byte typeB = 0;
                byte font = 0;
                byte exrom = 0;

                // Get the type of printer and confirm if it supports MICR reading and if it is equipped with an MICR unit.
                if (this.objAPI.GetType(out typeA, out font, out exrom, out typeB) == ErrorCode.SUCCESS)
                {
                    if ((typeA & BITMASK_WITHMICR) == 0)
                    {
                        MessageBox.Show("MICR is not supported with this model.",
                                        "Program05",
                                        MessageBoxButtons.OK,
                                        MessageBoxIcon.Exclamation);
                        retVal = false;
                    }
                    if ((typeB & BITMASK_MICREQUIPPED) == 0)
                    {
                        MessageBox.Show("MICR is not installed.",
                                        "Program05",
                                        MessageBoxButtons.OK,
                                        MessageBoxIcon.Exclamation);
                        retVal = false;
                    }
                }
                else
                {
                    retVal = false;
                }
            }

            // Set the read callback functions for print.
            if (retVal)
            {
                // Associate the created callback function to the event handler of StatusAPI.
                this.objAPI.StatusCallback += new StatusAPI.StatusCallbackHandler(CBPrinterStatus);

                if (this.objAPI.SetStatusBack() != ErrorCode.SUCCESS)
                {
                    MessageBox.Show("Failed to set callback function.",
                                    "Program05",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                    retVal = false;
                }
                else
                {
                    this.needCancelStatusBack = true;
                }
            }

            // Set the callback function for MICR.
            // The callback function is executed when MICR reading is complete.
            if (retVal)
            {
                // Associate the created callback function to the event handler of StatusAPI.
                this.objAPI.MICRReadCallback += new StatusAPI.MICRReadCallbackHandler(CBMICRRead);

                if (this.objAPI.MICRSetReadBack() != ErrorCode.SUCCESS)
                {
                    MessageBox.Show("Failed to set callback function.",
                                    "Program05",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                    retVal = false;
                }
                else
                {
                    this.needMICRCancelReadBack = true;
                }
            }

            // Select the data handling. See the manual for the valid parameters.
            // For this example, the following parameters are used
            //     charSel    : MF_CONTINUE_ANALYSIS (Replace with ?)
            //     detailSel  : MF_DETAILINFO_NOTADDED (No detailed information)
            //     errorSel   : ES_CONTINUE_ALL (Continue operation even with error)
            if (retVal)
            {
                if (this.objAPI.MICRSelectDataHandling(CharSelect.MF_CONTINUE_ANALYSIS, DetailSelect.MF_DETAILINFO_NOTADDED, ErrorSelect.ES_CONTINUE_ALL) != ErrorCode.SUCCESS)
                {
                    retVal = false;
                }
            }

            if (retVal)
            {
                this.isInitialized = true;
            }
            else
            {
                MessageBox.Show("Error occurred during initialization.",
                                "Program05",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation);
            }
            return retVal;
        }

        private void EndOperation()
        {
            if (this.needMICRCancelReadBack)
            {
                if (this.objAPI.MICRCancelReadBack() != ErrorCode.SUCCESS)
                {
                    MessageBox.Show("Failed to cancel read callback.",
                                    "Program05",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                }
                else
                {
                    this.needMICRCancelReadBack = false;
                }
            }

            if (this.needCancelStatusBack)
            {
                if (this.objAPI.CancelStatusBack() != ErrorCode.SUCCESS)
                {
                    MessageBox.Show("Failed to cancel read callback.",
                                    "Program05",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                }
                else
                {
                    this.needCancelStatusBack = false;
                }
            }

            // Always close the Status Monitor after using the Status API.
            if (this.needCloseMonPrinter)
            {
                if (this.objAPI.CloseMonPrinter() != ErrorCode.SUCCESS)
                {
                    MessageBox.Show("Failed to close printer status monitor.",
                                    "Program05",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                }
                else
                {
                    this.isInitialized = false;
                    this.needCloseMonPrinter = false;
                }
            }
        }

        // The callback function that will monitor printer/printing status.
        private void CBPrinterStatus(ASB status)
        {
            if (status.ToString().Contains(ASB.ASB_NO_RESPONSE.ToString()) ||
                status.ToString().Contains(ASB.ASB_PRINT_SUCCESS.ToString()) ||
                status.ToString().Contains(ASB.ASB_COVER_OPEN.ToString()) ||
                status.ToString().Contains(ASB.ASB_AUTOCUTTER_ERR.ToString()) ||
                status.ToString().Contains(ASB.ASB_RECEIPT_END.ToString()))
            {
                this.printStatus = status;
                this.isPrintFinish = true;
            }
        }

        // The callback function that will monitor MICR scanning status.
        private void CBMICRRead(string data, MICRStatus status, MICRDetail detail)
        {
            this.MICRdata = data;
            this.MICRstatus = status;
            this.MICRdetail = detail;

            this.isReadFinish = true;
        }
    }
}