using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using com.epson.pos.driver;
using System.Drawing.Printing;

namespace Program14
{
    public partial class Form1 : Form
    {

        // Constant variable holding the Printer name.
        private const string PRINTER_NAME = "EPSON TM-T88V Receipt";
        
        // Variables/Objects.
        private StatusAPI m_objAPI;
        PrintDocument pdPrint;

        // The event handler function when the Program executes.
        public Form1()
        {
            InitializeComponent();

            // Initialize other program components.

            m_objAPI = new StatusAPI();
            tmrTimer.Tick += new EventHandler(tmrTimer_Tick);

            lblMessage.Visible = false;
            cmdCancel.Visible = false;
            tmrTimer.Interval = 200;
        }

        private void cmdPrint_Click(object sender, EventArgs e)
        {
            pdPrint = new PrintDocument();
            pdPrint.PrintPage += new PrintPageEventHandler(pdPrint_Print);

            // Change the printer to the indicated printer.
            pdPrint.PrinterSettings.PrinterName = PRINTER_NAME;

            try
            {
                // Open Printer Monitor of Status API.
                if (m_objAPI.OpenMonPrinter(OpenType.TYPE_PRINTER, PRINTER_NAME) == ErrorCode.SUCCESS)
                    // Start the Timer.
                    // The timer is used here to imitate threading.
                    // Threading is used here so that other functions will still be accessible while printing is being executed.
                    tmrTimer.Start();
                else
                    MessageBox.Show("Failed to open printer status monitor.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            catch
            {
                MessageBox.Show("Failed to open StatusAPI.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        // The event handler function when pdPrint.Print is called.
        // This is where the actual printing of sample data to the printer is made.
        private void pdPrint_Print(object sender, PrintPageEventArgs e)
        {
            Font printFont = new Font("Lucida Console", (float)8, FontStyle.Regular, GraphicsUnit.Point); // Substituted to FontA Font

            e.Graphics.PageUnit = GraphicsUnit.Point;

            e.Graphics.DrawString("Hello APD!", printFont, Brushes.Black, 6, 4);

            // Indicate that no more data to print, and the Print Document can now send the print data to the spooler.
            e.HasMorePages = false;
        }

        // The event handler function when Timer is started.
        // This will first lock the printer for exclusive use.
        // If it can't lock the printer, it means that other program is still accessing the printer and locked it.
        // If that happens, the program will show the Cancel button so that the user can cancel its printing,
        // or the program will try to lock the printer from time to time.
        private void tmrTimer_Tick(object sender, EventArgs e)
        {
            // Lock the printer.
            if (m_objAPI.LockPrinter(10) == ErrorCode.SUCCESS)
                Print();
            else
            {
                // If the printer cannot be locked, display a message and the Cancel button.
                lblMessage.Visible = true;
                cmdCancel.Visible = true;
            }
        }

        private void Print()
        {
            // Stop the Timer, disable button and clear display.
            tmrTimer.Stop();
            tmrTimer.Dispose();
            lblMessage.Visible = false;
            cmdCancel.Visible = false;

            if (pdPrint.PrinterSettings.IsValid)
            {
                pdPrint.DocumentName = "Testing";
                // Start printing.
                pdPrint.Print();

                // Unlock the printer from the program's exclusive use.
                if (m_objAPI.UnlockPrinter() == ErrorCode.SUCCESS)
                    ClosePrinter();
                else
                    MessageBox.Show("Failed to unlock printer.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                tmrTimer.Stop();
                MessageBox.Show("Printer is not available.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }

        private void ClosePrinter()
        {
            // Close Printer Monitor.
            if (m_objAPI.IsValid)
                if (m_objAPI.CloseMonPrinter() != ErrorCode.SUCCESS)
                    MessageBox.Show("Failed to close printer status monitor.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        // The executed function when the Cancel button is clicked.
        private void cmdCancel_Click(object sender, EventArgs e)
        {
            // Stop the Timer, disable button and clear display.
            tmrTimer.Stop();
            lblMessage.Visible = false;
            cmdCancel.Visible = false;

            MessageBox.Show("Print job cancelled.", "Program14", MessageBoxButtons.OK, MessageBoxIcon.Information);

            ClosePrinter();
        }

        private void cmdClose_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}